@@ -11,8 +11,7 @@
 #   import pyflyby
 #   pyflyby.enable_auto_importer()
 
-from __future__ import (absolute_import, division, print_function,
-                        with_statement)
+
 
 from   pyflyby._interactive     import enable_auto_importer
 
