@@ -56,41 +56,42 @@ from   numpy                    import (Inf, NAN, NaN, abs as aabs, absolute,
                                         fromiter, frompyfunc, fromregex,
                                         fromstring, gradient, greater,
                                         greater_equal, hamming, hanning,
-                                        histogram, histogram2d, histogramdd,
-                                        hsplit, hstack, hypot, i0, identity,
-                                        iinfo, imag, in1d, index_exp, indices,
-                                        inexact, inf, inner, int0, int16,
-                                        int32, int64, int8, int_, intc,
-                                        integer, interp, intersect1d, intp,
-                                        invert, ipmt, irr, iscomplex,
-                                        iscomplexobj, isfinite, isfortran,
-                                        isinf, isnan, isneginf, isposinf,
-                                        isreal, isrealobj, isscalar, issctype,
-                                        issubclass_, issubdtype, issubsctype,
-                                        iterable, ix_, kaiser, kron, ldexp,
-                                        left_shift, less, less_equal, lexsort,
-                                        linalg, linspace, little_endian,
-                                        loadtxt, log, log as logarithm, log10,
-                                        log1p, log2, logaddexp, logaddexp2,
-                                        logical_and, logical_not, logical_or,
-                                        logical_xor, logspace, longcomplex,
-                                        longdouble, longfloat, longlong,
-                                        mafromtxt, mask_indices, mat, matrix,
-                                        maximum, mean, median, memmap,
-                                        meshgrid, mgrid, minimum, mintypecode,
-                                        mirr, mod, modf, msort, multiply, nan,
-                                        nan_to_num, nanargmax, nanargmin,
-                                        nanmax, nanmin, nansum, nbytes,
-                                        ndarray, ndenumerate, ndim, ndindex,
-                                        negative, newaxis, newbuffer,
-                                        nextafter, nonzero, not_equal, nper,
-                                        npv, number, object0, object_, ogrid,
-                                        ones, ones_like, outer, packbits, pi,
-                                        piecewise, pkgload, place, pmt, poly,
-                                        poly1d, polyadd, polyder, polydiv,
-                                        polyfit, polyint, polymul, polynomial,
-                                        polysub, polyval, power, ppmt, prod,
-                                        product, ptp, putmask, pv, r_, rad2deg,
+                                        heaviside, histogram, histogram2d,
+                                        histogramdd, hsplit, hstack, hypot,
+                                        i0, identity, iinfo, imag, in1d,
+                                        index_exp, indices, inexact, inf,
+                                        inner, int0, int16, int32, int64,
+                                        int8, int_, intc, integer, interp,
+                                        intersect1d, intp, invert, ipmt, irr,
+                                        iscomplex, iscomplexobj, isfinite,
+                                        isfortran, isinf, isnan, isneginf,
+                                        isposinf, isreal, isrealobj, isscalar,
+                                        issctype, issubclass_, issubdtype,
+                                        issubsctype, iterable, ix_, kaiser,
+                                        kron, ldexp, left_shift, less,
+                                        less_equal, lexsort, linalg, linspace,
+                                        little_endian, loadtxt, log,
+                                        log as logarithm, log10, log1p, log2,
+                                        logaddexp, logaddexp2, logical_and,
+                                        logical_not, logical_or, logical_xor,
+                                        logspace, longcomplex, longdouble,
+                                        longfloat, longlong, mafromtxt,
+                                        mask_indices, mat, matrix, maximum,
+                                        mean, median, memmap, meshgrid, mgrid,
+                                        minimum, mintypecode, mirr, mod, modf,
+                                        msort, multiply, nan, nan_to_num,
+                                        nanargmax, nanargmin, nanmax, nanmin,
+                                        nansum, nbytes, ndarray, ndenumerate,
+                                        ndim, ndindex, negative, newaxis,
+                                        newbuffer, nextafter, nonzero,
+                                        not_equal, nper, npv, number, object0,
+                                        object_, ogrid, ones, ones_like,
+                                        outer, packbits, pi, piecewise,
+                                        pkgload, place, pmt, poly, poly1d,
+                                        polyadd, polyder, polydiv, polyfit,
+                                        polyint, polymul, polynomial, polysub,
+                                        polyval, power, ppmt, prod, product,
+                                        ptp, putmask, pv, r_, rad2deg,
                                         radians, rank, rate, ravel, real,
                                         real_if_close, recarray, recfromcsv,
                                         recfromtxt, reciprocal, record,
