@@ -1,10 +1,9 @@
 # pyflyby/_imports2s.py.
 # Copyright (C) 2011-2018 Karl Chen.
 # License: MIT http://opensource.org/licenses/MIT
+from collections import defaultdict
 
-from __future__ import (absolute_import, division, print_function,
-                        with_statement)
-
+import isort
 from   pyflyby._autoimp         import scan_for_import_issues
 from   pyflyby._file            import FileText, Filename
 from   pyflyby._flags           import CompilerFlags
@@ -535,6 +534,65 @@ def replace_star_imports(codeblock, params=None):
     return transformer.output(params=params)
 
 
+def sort_imports(codeblock):
+    """
+    Sort imports for better grouping.
+    :param codeblock:
+    :return: codeblock
+    """
+    sorted_imports = isort.code(
+        str(codeblock),
+        # To sort all the import in lexicographic order
+        force_sort_within_sections=True,
+        # This is done below
+        lines_between_sections=0,
+        lines_after_imports=1
+    )
+    # Step 1: Split the input string into a list of lines
+    lines = sorted_imports.split('\n')
+
+    # Step 2: Identify groups of imports and keep track of their line numbers
+    pkg_lines = defaultdict(list)
+    line_pkg_dict = {}
+    for i, line in enumerate(lines):
+        match = re.match(r'(from (\w+)|import (\w+))', line)
+        if match:
+            current_pkg = match.groups()[1:3]
+            current_pkg = current_pkg[0] if current_pkg[0] is not None else current_pkg[1]
+            pkg_lines[current_pkg].append(i)
+            line_pkg_dict[i] = current_pkg
+
+    # Step 3: Create the output list of lines with blank lines around groups with more than one import
+    output_lines = []
+
+    def next_line(index):
+        if index + 1 < len(lines):
+            return lines[index + 1]
+        else:
+            return ""
+
+    for i, line in enumerate(lines):
+        if (
+            i > 0
+            and line_pkg_dict.get(i) != line_pkg_dict.get(i - 1)
+            and len(pkg_lines[line_pkg_dict.get(i)]) > 1
+            and next_line(i).startswith(("import", "from"))
+        ):
+            output_lines.append("")
+        output_lines.append(line)
+        if (
+            i < len(lines) - 1
+            and line_pkg_dict.get(i) != line_pkg_dict.get(i + 1)
+            and len(pkg_lines[line_pkg_dict.get(i)]) > 1
+            and next_line(i).startswith(("import", "from"))
+        ):
+            output_lines.append("")
+
+    # Step 4: Join the lines to create the output string
+    sorted_output_str = '\n'.join(output_lines)
+    return PythonBlock(sorted_output_str)
+
+
 def transform_imports(codeblock, transformations, params=None):
     """
     Transform imports as specified by ``transformations``.
