@@ -2,8 +2,7 @@
 # Copyright (C) 2011, 2012, 2013, 2014 Karl Chen.
 # License: MIT http://opensource.org/licenses/MIT
 
-from __future__ import (absolute_import, division, print_function,
-                        with_statement)
+
 
 import ast
 from   collections              import namedtuple
@@ -16,6 +15,41 @@ from   pyflyby._parse           import PythonStatement
 from   pyflyby._util            import (Inf, cached_attribute, cmp,
                                         longest_common_prefix)
 
+from   black                    import (find_pyproject_toml, format_str,
+                                        parse_pyproject_toml, TargetVersion,
+                                        FileMode as Mode)
+
+
+def read_black_config():
+    """Read the black configuration from ``pyproject.toml``
+
+
+    """
+    value = find_pyproject_toml('.')
+
+    raw_config = parse_pyproject_toml(value)
+
+    config = {}
+    for key in [
+        "line_length",
+        "skip_magic_trailing_comma",
+        "skip_string_normalization",
+    ]:
+        if key in raw_config:
+            config[key] = raw_config[key]
+    if "target_version" in raw_config:
+        target_version = raw_config["target_version"]
+        if isinstance(target_version, str):
+            config["target_version"] = target_version
+        elif isinstance(target_version, list):
+            # Convert TOML list to a Python set
+            config["target_version"] = set(target_version)
+        else:
+            raise ValueError(
+                f"Invalid config for black = {target_version!r} in {value}"
+            )
+    return config
+
 
 class ImportFormatParams(FormatParams):
     align_imports = True
@@ -103,10 +137,8 @@ class Import(object):
 
     @classmethod
     def from_parts(cls, fullname, import_as):
-        if not isinstance(fullname, str):
-            raise TypeError
-        if not isinstance(import_as, str):
-            raise TypeError
+        assert isinstance(fullname, str)
+        assert isinstance(import_as, str)
         self = object.__new__(cls)
         self.fullname = fullname
         self.import_as = import_as
@@ -312,6 +344,16 @@ class Import(object):
             return NotImplemented
         return self._data < other._data
 
+def _validate_alias(arg):
+    """
+    Ensure each alias is a tuple (str, None|str), and return it.
+
+    """
+    assert isinstance(arg, tuple)
+    a0,a1 = arg
+    assert isinstance(a0, str)
+    assert isinstance(a1, (str, type(None)))
+    return arg
 
 @total_ordering
 class ImportStatement(object):
@@ -336,23 +378,12 @@ class ImportStatement(object):
 
     @classmethod
     def from_parts(cls, fromname, aliases):
+        assert isinstance(aliases, list)
+        assert len(aliases) > 0
+        
         self = object.__new__(cls)
         self.fromname = fromname
-        if not len(aliases):
-            raise ValueError
-        def interpret_alias(arg):
-            if isinstance(arg, str):
-                return (arg, None)
-            if not isinstance(arg, tuple):
-                raise TypeError
-            if not len(arg) == 2:
-                raise TypeError
-            if not isinstance(arg[0], str):
-                raise TypeError
-            if not (arg[1] is None or isinstance(arg[1], str)):
-                raise TypeError
-            return arg
-        self.aliases = tuple(interpret_alias(a) for a in aliases)
+        self.aliases = tuple(_validate_alias(a) for a in aliases)
         return self
 
     @classmethod
@@ -393,20 +424,18 @@ class ImportStatement(object):
           `ImportStatement`
         """
         if isinstance(node, ast.ImportFrom):
-            if isinstance(node.module, str):
-                module = node.module
-            elif node.module is None:
-                # In python2.7, ast.parse("from . import blah") yields
-                # node.module = None.  In python2.6, it's the empty string.
+            if node.module is None:
                 module = ''
             else:
-                raise TypeError("unexpected node.module=%s"
-                                % type(node.module).__name__)
+                assert isinstance(node.module, str)
+                module = node.module
             fromname = '.' * node.level + module
         elif isinstance(node, ast.Import):
             fromname = None
         else:
-            raise NonImportStatementError
+            raise NonImportStatementError(
+                    'Expected ImportStatement, got {node}'.format(node=node)
+                    )
         aliases = [ (alias.name, alias.asname) for alias in node.names ]
         return cls.from_parts(fromname, aliases)
 
@@ -427,7 +456,7 @@ class ImportStatement(object):
             raise ValueError
         module_names = set(imp.split.module_name for imp in imports)
         if len(module_names) > 1:
-            raise Exception(
+            raise ValueError(
                 "Inconsistent module names %r" % (sorted(module_names),))
         fromname = list(module_names)[0]
         aliases = [ imp.split[1:] for imp in imports ]
@@ -492,11 +521,58 @@ class ImportStatement(object):
             tokens.append(t)
         res = s0 + pyfill(s, tokens, params=params)
         if params.use_black:
-            import black
-            mode = black.FileMode()
-            return black.format_str(res, mode=mode)
+            return self.run_black(res, params)
         return res
 
+    @staticmethod
+    def run_black(src_contents: str, params) -> str:
+        """Run the black formatter for the Python source code given as a string
+
+        This is adapted from https://github.com/akaihola/darker
+
+        """
+        black_config = read_black_config()
+        mode = dict()
+        if "line_length" in black_config:
+            mode["line_length"] = (
+                params.max_line_length
+                if params.max_line_length
+                else black_config["line_length"]
+            )
+        if "target_version" in black_config:
+            if isinstance(black_config["target_version"], set):
+                target_versions_in = black_config["target_version"]
+            else:
+                target_versions_in = {black_config["target_version"]}
+            all_target_versions = {
+                tgt_v.name.lower(): tgt_v for tgt_v in TargetVersion
+            }
+            bad_target_versions = target_versions_in - set(all_target_versions)
+            if bad_target_versions:
+                raise ValueError(
+                    f"Invalid target version(s) {bad_target_versions}"
+                )
+            mode["target_versions"] = {
+                all_target_versions[n] for n in target_versions_in
+            }
+        if "skip_magic_trailing_comma" in black_config:
+            mode["magic_trailing_comma"] = not black_config[
+                "skip_magic_trailing_comma"
+            ]
+        if "skip_string_normalization" in black_config:
+            # The ``black`` command line argument is
+            # ``--skip-string-normalization``, but the parameter for
+            # ``black.Mode`` needs to be the opposite boolean of
+            # ``skip-string-normalization``, hence the inverse boolean
+            mode["string_normalization"] = not black_config[
+                "skip_string_normalization"
+            ]
+
+        # The custom handling of empty and all-whitespace files below will be unnecessary if
+        # https://github.com/psf/black/pull/2484 lands in Black.
+        contents_for_black = src_contents
+        return format_str(contents_for_black, mode=Mode(**mode))
+
     @property
     def _data(self):
         return (self.fromname, self.aliases)
