@@ -2,8 +2,7 @@
 # Copyright (C) 2011, 2012, 2013, 2014, 2015, 2018 Karl Chen.
 # License: MIT http://opensource.org/licenses/MIT
 
-from __future__ import (absolute_import, division, print_function,
-                        with_statement)
+
 
 import ast
 from   collections              import namedtuple
@@ -16,7 +15,6 @@ from   textwrap                 import dedent
 import types
 
 import six
-from   six                      import PY2, PY3, text_type as unicode
 from   six.moves                import range
 
 from   pyflyby._file            import FilePos, FileText, Filename
@@ -24,25 +22,9 @@ from   pyflyby._flags           import CompilerFlags
 from   pyflyby._log             import logger
 from   pyflyby._util            import cached_attribute, cmp
 
-if PY3:
-    from ast import Bytes
-else:
-    Bytes = ast.Str
+from ast import Bytes
 
-
-if sys.version_info >= (3, 8):
-    from ast import TypeIgnore, AsyncFunctionDef
-else:
-
-    # TypeIgnore, AsyncFunctionDef does not exist on Python 3.7 and before. thus
-    # we define a dummy TypeIgnore, AsyncFunctionDef just to simplify remaining
-    # code.
-
-    class TypeIgnore:
-        pass
-
-    class AsyncFunctionDef:
-        pass
+from ast import TypeIgnore, AsyncFunctionDef
 
 
 def _is_comment_or_blank(line):
@@ -110,43 +92,27 @@ def _iter_child_nodes_in_order_internal_1(node):
         assert node._fields == ("keys", "values")
         yield list(zip(node.keys, node.values))
     elif isinstance(node, (ast.FunctionDef, AsyncFunctionDef)):
-        if six.PY2:
-            assert node._fields == ('name', 'args', 'body', 'decorator_list'), node._fields
-            yield node.decorator_list, node.args, node.body
-        elif sys.version_info >= (3, 8):
-            assert node._fields == (
-                "name",
-                "args",
-                "body",
-                "decorator_list",
-                "returns",
-                "type_comment",
-            ), node._fields
-            res = (
-                node.type_comment,
-                node.decorator_list,
-                node.args,
-                node.returns,
-                node.body,
-            )
-            yield res
-        else:
-            assert node._fields == ('name', 'args', 'body', 'decorator_list',
-                                    'returns'), node._fields
-            yield node.decorator_list, node.args, node.returns, node.body
+        assert node._fields == (
+            "name",
+            "args",
+            "body",
+            "decorator_list",
+            "returns",
+            "type_comment",
+        ), node._fields
+        res = (
+            node.type_comment,
+            node.decorator_list,
+            node.args,
+            node.returns,
+            node.body,
+        )
+        yield res
         # node.name is a string, not an AST node
     elif isinstance(node, ast.arguments):
-        if six.PY2:
-            assert node._fields == ('args', 'vararg', 'kwarg', 'defaults'), node._fields
-            args = node.args
-        elif sys.version_info >= (3, 8):
-            assert node._fields == ('posonlyargs', 'args', 'vararg', 'kwonlyargs',
-                                    'kw_defaults', 'kwarg', 'defaults'), node._fields
-            args = node.posonlyargs + node.args
-        else:
-            assert node._fields == ('args', 'vararg', 'kwonlyargs',
-                                    'kw_defaults', 'kwarg', 'defaults'), node._fields
-            args = node.args
+        assert node._fields == ('posonlyargs', 'args', 'vararg', 'kwonlyargs',
+                                'kw_defaults', 'kwarg', 'defaults'), node._fields
+        args = node.posonlyargs + node.args
         defaults = node.defaults or ()
         num_no_default = len(args) - len(defaults)
         yield args[:num_no_default]
@@ -162,13 +128,10 @@ def _iter_child_nodes_in_order_internal_1(node):
                       [(k.lineno,k.col_offset, k) for k in node.args])
         yield [a[2] for a in args]
     elif isinstance(node, ast.ClassDef):
-        if six.PY2:
-            assert node._fields == ('name', 'bases', 'body', 'decorator_list')
-        else:
-            assert node._fields == ('name', 'bases', 'keywords', 'body', 'decorator_list')
+        assert node._fields == ('name', 'bases', 'keywords', 'body', 'decorator_list')
         yield node.decorator_list, node.bases, node.body
         # node.name is a string, not an AST node
-    elif sys.version_info >= (3, 7) and isinstance(node, ast.FormattedValue):
+    elif isinstance(node, ast.FormattedValue):
         assert node._fields == ('value', 'conversion', 'format_spec')
         yield node.value,
     else:
@@ -202,25 +165,10 @@ def _flags_to_try(source, flags, auto_flags, mode):
     If ``auto_flags`` is True, then yield ``flags`` and ``flags ^ print_function``.
     """
     flags = CompilerFlags(flags)
-    if sys.version_info >= (3, 8):
-        if re.search(r"# *type:", source):
-            flags = flags | CompilerFlags('type_comments')
-        yield flags
-        return
-    if not auto_flags:
-        yield flags
-        return
-    if PY3:
-        yield flags
-        return
-    if mode == "eval":
-        if re.search(r"\bprint\b", source):
-            flags = flags | CompilerFlags("print_function")
-        yield flags
-        return
+    if re.search(r"# *type:", source):
+        flags = flags | CompilerFlags('type_comments')
     yield flags
-    if re.search(r"\bprint\b", source):
-        yield flags ^ CompilerFlags("print_function")
+    return
 
 
 def _parse_ast_nodes(text, flags, auto_flags, mode):
@@ -248,8 +196,6 @@ def _parse_ast_nodes(text, flags, auto_flags, mode):
     filename = str(text.filename) if text.filename else "<unknown>"
     source = text.joined
     source = dedent(source)
-    if PY2 and isinstance(source, unicode):
-        source = source.encode('utf-8')
     if not source.endswith("\n"):
         # Ensure that the last line ends with a newline (``ast`` barfs
         # otherwise).
@@ -283,12 +229,6 @@ def _test_parse_string_literal(text, flags):
 
     """
     text = FileText(text)
-    if PY2:
-        try:
-            text.joined.encode('ascii')
-        except UnicodeError:
-            text = FileText(u'# encoding: utf-8\n' + unicode(text), filename=text.filename)
-
     try:
         module_node = _parse_ast_nodes(text, flags, False, "eval")
     except SyntaxError:
@@ -374,18 +314,6 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
     """
     assert isinstance(ast_node, (ast.AST, str, TypeIgnore)), ast_node
 
-    # joined strings and children do not carry a column offset on pre-3.8
-    # this prevent reformatting.
-    # set the column offset to the parent value before 3.8
-    if (3, 7) < sys.version_info < (3, 8):
-        if (
-            isinstance(ast_node, (getattr(ast, "JoinedStr", None), ast.FormattedValue))
-            or isinstance(
-                parent_ast_node, (getattr(ast, "JoinedStr", None), ast.FormattedValue)
-            )
-        ) and ast_node.col_offset == -1:
-            ast_node.col_offset = parent_ast_node.col_offset
-
     # First, traverse child nodes.  If the first child node (recursively) is a
     # multiline string, then we need to transfer its information to this node.
     # Walk all nodes/fields of the AST.  We implement this as a custom
@@ -430,9 +358,8 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
     if ast_node.col_offset >= 0:
         # In Python 3.8+, FunctionDef.lineno is the line with the def. To
         # account for decorators, we need the lineno of the first decorator
-        if (sys.version_info >= (3, 8)
-            and isinstance(ast_node, (ast.FunctionDef, ast.ClassDef, AsyncFunctionDef))
-            and ast_node.decorator_list):
+        if (isinstance(ast_node, (ast.FunctionDef, ast.ClassDef, AsyncFunctionDef))
+                and ast_node.decorator_list):
             delta = (ast_node.decorator_list[0].lineno-1,
                      # The col_offset doesn't include the @
                      ast_node.decorator_list[0].col_offset - 1)
@@ -447,10 +374,6 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
         # Special case for 'with' statements.  Consider the code:
         #    with X: pass
         #    ^0   ^5
-        # In python2.6, col_offset is 0.
-        # In python2.7, col_offset is 5.
-        # This is because python2.7 allows for multiple clauses:
-        #    with X, Y: pass
         # Since 'Y's col_offset isn't the beginning of the line, the authors
         # of Python presumably changed 'X's col_offset to also not be the
         # beginning of the line.  If they had made the With ast node support
@@ -460,20 +383,6 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
         # Since we use startpos for breaking lines, we need to set startpos to
         # the beginning of the line.
         # In Python 3, the col_offset for the with is 0 again.
-        if (isinstance(ast_node, ast.With) and
-            not isinstance(parent_ast_node, ast.With) and
-            sys.version_info[:2] == (2,7)):
-            assert ast_node.col_offset >= 5
-            if startpos.lineno == text.startpos.lineno:
-                linestart = text.startpos.colno
-            else:
-                linestart = 1
-            line = text[(startpos.lineno,linestart):startpos]
-            m = re.search(r"\bwith\s+$", str(line))
-            assert m
-            lk = len(m.group()) # length of 'with   ' including spaces
-            startpos = FilePos(startpos.lineno, startpos.colno - lk)
-            assert str(text[startpos:(startpos+(0,4))]) == "with"
         ast_node.startpos = startpos
         if sys.version_info <= (3, 8):
             ast_node.startpos = max(startpos, minpos)
@@ -506,7 +415,7 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
 
     # It should now be the case that we are looking at a multi-line string
     # literal.
-    if sys.version_info >= (3, 7) and isinstance(ast_node, ast.FormattedValue):
+    if isinstance(ast_node, ast.FormattedValue):
         ast_node.startpos = ast_node.value.startpos
         ast_node.endpos = ast_node.value.startpos
 
@@ -534,9 +443,6 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
             for _m in re.finditer("[bBrRuU]*[\"\']", start_line)])
     target_str = ast_node.s
 
-    if isinstance(target_str, bytes) and sys.version_info[:2] == (3, 7):
-        target_str = target_str.decode()
-
     # Loop over possible end_linenos.  The first one we've identified is the
     # by far most likely one, but in theory it could be anywhere later in the
     # file.  This could be because of a dastardly concatenated string like
@@ -601,18 +507,8 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
             candidate_str = _test_parse_string_literal(subtext, flags)
             if candidate_str is None:
                 continue
-            if isinstance(candidate_str, bytes) and sys.version_info[:2] == (3, 7):
-                candidate_str = candidate_str.decode()
 
             maybe_fstring = False
-            try:
-                if (3, 7) <= sys.version_info <= (3, 8):
-                    potential_start = text.lines[startpos.lineno - 1]
-                    maybe_fstring = ("f'" in potential_start) or (
-                        'f"' in potential_start
-                    )
-            except IndexError:
-                pass
 
             if target_str == candidate_str and target_str:
                 # Success!
@@ -637,17 +533,6 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
             for (sq, sp) in startpos_candidates
             if sp in matched_prefix
         ]
-    if (3, 7) <= sys.version_info <= (3, 8):
-        if len(f_string_candidate_prefixes) == 1:
-            # we did not find the string but there is one fstring candidate starting it
-
-            ast_node.startpos, ast_node.endpos = f_string_candidate_prefixes[0]
-            return True
-        elif isinstance(parent_ast_node, ast.JoinedStr):
-            self_pos = parent_ast_node.values.index(ast_node)
-            ast_node.startpos = parent_ast_node.values[self_pos - 1].startpos
-            ast_node.endpos = parent_ast_node.values[self_pos - 1].endpos
-            return True
     raise ValueError("Couldn't find exact position of %s" % (ast.dump(ast_node)))
 
 
@@ -835,7 +720,7 @@ class PythonStatement(object):
     comments/blank lines.
 
       >>> PythonStatement('print("x",\n file=None)\n', flags='print_function')  #doctest: +SKIP
-      PythonStatement('print("x",\n file=None)\n', flags=0x10000)
+      PythonStatement('print("x",\n file=None)\n', flags=0x100000)
 
     Implemented as a wrapper around a `PythonBlock` containing at most one
     top-level AST node.
