@@ -4,8 +4,7 @@
 
 # Deprecated stub for backwards compatibility.
 
-from __future__ import (absolute_import, division, print_function,
-                        with_statement)
+
 
 from   pyflyby._importdb        import ImportDB
 
