@@ -3,8 +3,7 @@
 # License for THIS FILE ONLY: CC0 Public Domain Dedication
 # http://creativecommons.org/publicdomain/zero/1.0/
 
-from __future__ import (absolute_import, division, print_function,
-                        with_statement)
+
 
 import ast
 import os
@@ -14,8 +13,6 @@ import sys
 from   tempfile                 import mkdtemp
 from   textwrap                 import dedent
 
-from   six                      import PY2, PY3
-
 from   pyflyby                  import (Filename, ImportDB, auto_eval,
                                         auto_import, find_missing_imports)
 from   pyflyby._autoimp         import (LoadSymbolError, load_symbol,
@@ -23,6 +20,7 @@ from   pyflyby._autoimp         import (LoadSymbolError, load_symbol,
 from   pyflyby._idents          import DottedIdentifier
 from   pyflyby._importstmt      import Import
 from   pyflyby._flags           import CompilerFlags
+from pyflyby._util import CwdCtx
 
 
 @pytest.fixture
@@ -147,10 +145,7 @@ def test_find_missing_imports_lambda_3():
 def test_find_missing_imports_list_comprehension_1():
     result   = find_missing_imports("[x+y+z for x,y in [(1,2)]], y", [{}])
     result   = _dilist2strlist(result)
-    if PY2:
-        expected = ['z']
-    else:
-        expected = ['y', 'z']
+    expected = ['y', 'z']
     assert expected == result
 
 
@@ -278,9 +273,7 @@ def test_find_missing_imports_function_defaults_kwargs_1():
     expected = ['args', 'kwargs', 'y']
     assert expected == result
     
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
+
 def test_find_missing_imports_kwarg_annotate():
     """
     pfb issue 162
@@ -528,20 +521,58 @@ def test_method_reference_current_class():
     assert unused == []
 
 
+def test_annotation_inside_class():
+    code = dedent(
+        """
+        class A:
+            param1: str
+            param2: B
 
+        class B:
+            param1: str
+   """
+    )
+    missing, unused = scan_for_import_issues(code, [{}])
+    assert missing == []
+    assert unused == []
+
+
+@pytest.mark.xfail(
+    reason="Had to deactivate as part of https://github.com/deshaw/pyflyby/pull/269/files conflicting requirements"
+)
 def test_find_missing_imports_class_name_1():
     code = dedent(
         """
-        class Corinne(object):
+        class Corinne:
             pass
-        class Bobtail(object):
-            class Chippewa(object):
-                Bobtail
+        class Bobtail:
+            class Chippewa:
+                Bobtail # will be name error at runtime
             Rockton = Passall, Corinne, Chippewa
-    """)
-    result   = find_missing_imports(code, [{}])
-    result   = _dilist2strlist(result)
-    expected = ['Bobtail', 'Passall']
+                      # ^error, ^ok   , ^ok
+    """
+    )
+    result = find_missing_imports(code, [{}])
+    result = _dilist2strlist(result)
+    expected = ["Bobtail", "Passall"]
+    assert expected == result
+
+
+def test_find_missing_imports_class_name_1b():
+    code = dedent(
+        """
+        class Corinne:
+            pass
+        class Bobtail:
+            class Chippewa:
+                Bobtail # will be name error at runtime
+            Rockton = Passall, Corinne, Chippewa
+                      # ^error, ^ok   , ^ok
+    """
+    )
+    result = find_missing_imports(code, [{}])
+    result = _dilist2strlist(result)
+    expected = ["Passall"]
     assert expected == result
 
 
@@ -723,10 +754,7 @@ def test_find_missing_imports_class_member_generator_expression_1():
     """)
     result   = find_missing_imports(code, [{}])
     result   = _dilist2strlist(result)
-    if PY2:
-        expected = ['y1']
-    else:
-        expected = ['y1', 'y2']
+    expected = ['y1', 'y2']
     assert expected == result
 
 
@@ -868,14 +896,9 @@ def test_find_missing_imports_global_1():
 
 
 def test_find_missing_imports_complex_1():
-    if PY2:
-        code = dedent("""
-            x = 3+4j+5L+k+u'a'
-        """)
-    else:
-        code = dedent("""
-            x = 3+4j+5+k+u'a'
-        """)
+    code = dedent("""
+        x = 3+4j+5+k+u'a'
+    """)
     result   = find_missing_imports(code, [{}])
     result   = _dilist2strlist(result)
     expected = ['k']
@@ -945,9 +968,7 @@ def test_find_missing_imports_code_loop_1():
     expected = ['use', 'y']
     assert expected == result
 
-@pytest.mark.skipif(
-    sys.version_info < (3, 8),
-    reason="Python 3.8+-only syntax.")
+
 def test_find_missing_imports_positional_only_args_1():
     code = dedent("""
         def func(x, /, y):
@@ -958,9 +979,7 @@ def test_find_missing_imports_positional_only_args_1():
     expected = []
     assert expected == result
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
+
 def test_find_missing_imports_keyword_only_args_1():
     code = dedent("""
         def func(*args, kwonly=b):
@@ -971,9 +990,7 @@ def test_find_missing_imports_keyword_only_args_1():
     expected = ['b']
     assert expected == result
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
+
 def test_find_missing_imports_keyword_only_args_2():
     code = dedent("""
         def func(*args, kwonly):
@@ -985,9 +1002,6 @@ def test_find_missing_imports_keyword_only_args_2():
     assert expected == result
 
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
 def test_find_missing_imports_keyword_only_args_3():
     code = dedent("""
         def func(*args, kwonly, kwonly2=b):
@@ -998,9 +1012,7 @@ def test_find_missing_imports_keyword_only_args_3():
     expected = ['b']
     assert expected == result
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
+
 def test_find_missing_imports_annotations_1():
     code = dedent("""
         def func(a: b) -> c:
@@ -1011,9 +1023,7 @@ def test_find_missing_imports_annotations_1():
     expected = ['b', 'c']
     assert expected == result
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
+
 def test_find_missing_imports_annotations_2():
     code = dedent("""
     a: b = c
@@ -1023,9 +1033,7 @@ def test_find_missing_imports_annotations_2():
     expected = ['b', 'c']
     assert expected == result
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
+
 def test_find_missing_imports_star_assignments_1():
     code = dedent("""
     a, *b = c
@@ -1035,9 +1043,7 @@ def test_find_missing_imports_star_assignments_1():
     expected = ['c']
     assert expected == result
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
+
 def test_find_missing_imports_star_expression_1():
     code = dedent("""
     [a, *b]
@@ -1048,9 +1054,6 @@ def test_find_missing_imports_star_expression_1():
     assert expected == result
 
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
 def test_find_missing_imports_star_expression_2():
     code = dedent("""
     {a: 1, **b, **{c: 1}}
@@ -1060,9 +1063,7 @@ def test_find_missing_imports_star_expression_2():
     expected = ['a', 'b', 'c']
     assert expected == result
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
+
 def test_find_missing_imports_star_expression_function_call_1():
     code = dedent("""
     f(a, *b, **c, d=e, **g)
@@ -1073,7 +1074,6 @@ def test_find_missing_imports_star_expression_function_call_1():
     assert expected == result
 
 def test_find_missing_imports_star_expression_function_call_2():
-    # Python 2-valid syntax
     code = dedent("""
     f(a, b=c, *d, **e)
     """)
@@ -1082,9 +1082,7 @@ def test_find_missing_imports_star_expression_function_call_2():
     expected = ['a', 'c', 'd', 'e', 'f']
     assert expected == result
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
+
 def test_find_missing_imports_python_3_metaclass_1():
     code = dedent("""
     class Test(metaclass=TestMeta):
@@ -1096,9 +1094,6 @@ def test_find_missing_imports_python_3_metaclass_1():
     assert expected == result
 
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
 def test_find_missing_imports_f_string_1():
     code = dedent("""
     a = 1
@@ -1109,9 +1104,7 @@ def test_find_missing_imports_f_string_1():
     expected = ['b']
     assert expected == result
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
+
 def test_find_missing_imports_f_string_2():
     code = dedent("""
     a = 1
@@ -1122,9 +1115,7 @@ def test_find_missing_imports_f_string_2():
     expected = ['b']
     assert expected == result
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
+
 def test_find_missing_imports_f_string_3():
     # Recursive format spec
     code = dedent("""
@@ -1155,9 +1146,6 @@ def test_find_missing_imports_true_false_none_1():
     assert expected == result
 
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
 def test_find_missing_imports_matmul_1():
     code = dedent("""
     a@b
@@ -1167,9 +1155,7 @@ def test_find_missing_imports_matmul_1():
     expected = ['a', 'b']
     assert expected == result
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
+
 def test_find_missing_imports_async_await_1():
     code = dedent("""
     async def f():
@@ -1182,9 +1168,7 @@ def test_find_missing_imports_async_await_1():
     expected = ['a', 'c', 'e', 'h']
     assert expected == result
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
+
 def test_find_missing_imports_async_comprehension_1():
     code = dedent("""
     async def f():
@@ -1196,9 +1180,6 @@ def test_find_missing_imports_async_comprehension_1():
     assert expected == result
 
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
 def test_find_missing_imports_yield_from_1():
     code = dedent("""
     def f():
@@ -1210,7 +1191,6 @@ def test_find_missing_imports_yield_from_1():
     assert expected == result
 
 
-@pytest.mark.skipif(PY2, reason="Python 3-only syntax.")
 def test_find_missing_imports_not_async_def():
     code = dedent(
         """
@@ -1225,7 +1205,6 @@ def test_find_missing_imports_not_async_def():
 
 
 def test_find_missing_imports_nested_with_1():
-    # Handled differently in the ast in Python 2 and 3
     code = dedent("""
     with a as b, c as d:
         pass
@@ -1282,11 +1261,6 @@ def test_find_missing_imports_tuple_ellipsis_type_1():
     assert expected == result
 
 
-# Only Python 3.8 includes type comments in the ast, so we only support this
-# there (see issue #31).
-@pytest.mark.skipif(
-    sys.version_info < (3, 8),
-    reason="Python 3.8+-only support.")
 def test_scan_for_import_issues_type_comment_1():
     code = dedent("""
     from typing import Sequence
@@ -1299,11 +1273,6 @@ def test_scan_for_import_issues_type_comment_1():
     assert missing == []
 
 
-# Only Python 3.8 includes type comments in the ast, so we only support this
-# there (see issue #31, 171, 174).
-@pytest.mark.skipif(
-    sys.version_info < (3, 8),
-    reason="Python 3.8+-only support.")
 def test_scan_for_import_issues_type_comment_2():
     code = dedent("""
     from typing import Sequence
@@ -1316,9 +1285,6 @@ def test_scan_for_import_issues_type_comment_2():
     assert missing == []
 
 
-@pytest.mark.skipif(
-    sys.version_info < (3, 8),
-    reason="Python 3.8+-only support.")
 def test_scan_for_import_issues_type_comment_3():
     code = dedent("""
     def foo(strings):
@@ -1330,9 +1296,6 @@ def test_scan_for_import_issues_type_comment_3():
     assert missing == [(1, DottedIdentifier('Sequence'))]
 
 
-@pytest.mark.skipif(
-    sys.version_info < (3, 8),
-    reason="Python 3.8+-only support.")
 def test_scan_for_import_issues_type_comment_4():
     code = dedent("""
     from typing import Sequence, Tuple
@@ -1344,12 +1307,7 @@ def test_scan_for_import_issues_type_comment_4():
     assert unused == [(2, Import('from typing import Tuple'))]
     assert missing == []
 
-# Python 3.8 uses the correct line number for multiline strings (the first
-# line), making _annotate_ast_startpos irrelevant. Otherwise, the logic for
-# getting this right is too hard. See issue #12.
-@pytest.mark.skipif(
-    sys.version_info < (3, 8),
-    reason="Python 3.8+-only support.")
+
 def test_scan_for_import_issues_multiline_string_1():
     code = dedent('''
     x = (
@@ -1685,6 +1643,20 @@ def test_scan_for_import_issues_setattr_in_func_1():
     # used.  (This was buggy before 201907.)
     assert unused == [(2, Import('import cc'))]
 
+
+def test_scan_for_import_issues_class_defined_after_use():
+    code = dedent("""
+    def foo():
+        Bar.counter = 1
+
+    class Bar:
+        counter = 0
+    """)
+    missing, unused = scan_for_import_issues(code)
+    assert missing == []
+    assert unused == []
+
+
 def test_setattr_is_not_unused():
     code = dedent("""
         from a import b
@@ -1846,28 +1818,15 @@ def test_auto_eval_exec_1():
 
 
 def test_auto_eval_no_auto_flags_ps_flagps_1(capsys):
-    if PY2:
-        auto_eval("print 3.00", flags=None, auto_flags=False)
-        out, _ = capsys.readouterr()
-        assert out == "3.0\n"
-    else:
-        auto_eval("print(3.00)", flags=CompilerFlags.from_int(0), auto_flags=False)
-        out, _ = capsys.readouterr()
-        assert out == "3.0\n"
+    auto_eval("print(3.00)", flags=CompilerFlags.from_int(0), auto_flags=False)
+    out, _ = capsys.readouterr()
+    assert out == "3.0\n"
 
 def test_auto_eval_no_auto_flags_ps_flag_pf1():
     with pytest.raises(SyntaxError):
         auto_eval("print 3.00", flags="print_function", auto_flags=False)
 
 
-@pytest.mark.skipif(
-    PY3,
-    reason="print function is not invalid syntax in Python 3.")
-def test_auto_eval_no_auto_flags_pf_flagps_1():
-    with pytest.raises(SyntaxError):
-        auto_eval("print(3.00, file=sys.stdout)", flags=None, auto_flags=False)
-
-
 def test_auto_eval_no_auto_flags_pf_flag_pf1(capsys):
     auto_eval("print(3.00, file=sys.stdout)",
               flags="print_function", auto_flags=False)
@@ -1876,21 +1835,8 @@ def test_auto_eval_no_auto_flags_pf_flag_pf1(capsys):
 
 
 def test_auto_eval_auto_flags_ps_flagps_1(capsys):
-    if PY2:
+    with pytest.raises(SyntaxError):
         auto_eval("print 3.00", flags=CompilerFlags.from_int(0), auto_flags=True)
-        out, _ = capsys.readouterr()
-        assert out == "3.0\n"
-    else:
-        with pytest.raises(SyntaxError):
-            auto_eval("print 3.00", flags=CompilerFlags.from_int(0), auto_flags=True)
-
-@pytest.mark.skipif(
-    PY3,
-    reason="print not as a function cannot be valid syntax in Python 3.")
-def test_auto_eval_auto_flags_ps_flag_pf1(capsys):
-    auto_eval("print 3.00", flags=CompilerFlags("print_function"), auto_flags=True)
-    out, _ = capsys.readouterr()
-    assert out == "3.0\n"
 
 
 def test_auto_eval_auto_flags_pf_flagps_1(capsys):
@@ -2026,18 +1972,11 @@ def test_auto_import_unknown_but_in_db1(tpp, capsys):
     db = ImportDB('import photon70447198')
     auto_import("photon70447198.asdfasdf", [{}], db=db)
     out, _ = capsys.readouterr()
-    if PY2:
-        expected = dedent("""
-            [PYFLYBY] import photon70447198
-            [PYFLYBY] Error attempting to 'import photon70447198': ImportError: No module named photon70447198
-            Traceback (most recent call last):
-        """).lstrip()
-    else:
-        expected = dedent("""
-            [PYFLYBY] import photon70447198
-            [PYFLYBY] Error attempting to 'import photon70447198': ModuleNotFoundError: No module named 'photon70447198'
-            Traceback (most recent call last):
-        """).lstrip()
+    expected = dedent("""
+        [PYFLYBY] import photon70447198
+        [PYFLYBY] Error attempting to 'import photon70447198': ModuleNotFoundError: No module named 'photon70447198'
+        Traceback (most recent call last):
+    """).lstrip()
 
     assert out.startswith(expected)
 
@@ -2062,18 +2001,11 @@ def test_auto_import_indirect_importerror_1(tpp, capsys):
     """)
     auto_import("neutron46291483.asdfasdf", [{}])
     out, _ = capsys.readouterr()
-    if PY2:
-        expected = dedent("""
-            [PYFLYBY] import neutron46291483
-            [PYFLYBY] Error attempting to 'import neutron46291483': ImportError: No module named baryon96446873
-            Traceback (most recent call last):
-        """).lstrip()
-    else:
-        expected = dedent("""
-            [PYFLYBY] import neutron46291483
-            [PYFLYBY] Error attempting to 'import neutron46291483': ModuleNotFoundError: No module named 'baryon96446873'
-            Traceback (most recent call last):
-        """).lstrip()
+    expected = dedent("""
+        [PYFLYBY] import neutron46291483
+        [PYFLYBY] Error attempting to 'import neutron46291483': ModuleNotFoundError: No module named 'baryon96446873'
+        Traceback (most recent call last):
+    """).lstrip()
 
     assert out.startswith(expected)
 
@@ -2122,3 +2054,25 @@ def test_post_import_hook_fullname():
     def test_hook(val):
         assert val == expected
     auto_import("numpy.compat", [{}], db=db, post_import_hook=test_hook)
+
+
+def test_namespace_package(tpp, capsys):
+    os.mkdir(str(tpp/'namespace_package'))
+    auto_import("namespace_package", [{}])
+    out, _ = capsys.readouterr()
+    expected = dedent("""
+        [PYFLYBY] import namespace_package
+    """).lstrip()
+    assert out.startswith(expected)
+
+
+def test_unsafe_filename_warning(tpp, capsys):
+    filepath = os.path.join(tpp._filename, 'foo#bar')
+    os.mkdir(filepath)
+    with CwdCtx(filepath):
+        auto_import("pyflyby", [{}])
+    out, _ = capsys.readouterr()
+    expected = dedent("""
+        [PYFLYBY] import pyflyby
+    """).lstrip()
+    assert out.startswith(expected)
