@@ -3,8 +3,7 @@
 # License for THIS FILE ONLY: CC0 Public Domain Dedication
 # http://creativecommons.org/publicdomain/zero/1.0/
 
-from __future__ import (absolute_import, division, print_function,
-                        with_statement)
+
 
 from   io                       import BytesIO
 import os
@@ -14,9 +13,7 @@ import subprocess
 import tempfile
 from   textwrap                 import dedent
 
-from   six                      import PY2, PY3
-
-from   pyflyby._util            import EnvVarCtx
+from pyflyby._util import EnvVarCtx, CwdCtx
 
 import pytest
 
@@ -26,12 +23,14 @@ BIN_DIR = os.path.join(PYFLYBY_HOME, "bin")
 
 python = sys.executable
 
-def pipe(command, stdin=""):
+def pipe(command, stdin="", cwd=None, env=None):
     return subprocess.Popen(
         [python] + command,
         stdin=subprocess.PIPE,
         stdout=subprocess.PIPE,
-        stderr=subprocess.STDOUT
+        stderr=subprocess.STDOUT,
+        cwd=cwd,
+        env=env
     ).communicate(stdin.encode('utf-8'))[0].decode('utf-8').strip()
 
 
@@ -122,6 +121,7 @@ def test_tidy_imports_filename_action_replace_1():
             foo() + os + sys
         import a
         import c
+
         a, c
     ''').lstrip()
     assert result == expected_result
@@ -139,6 +139,7 @@ def test_tidy_imports_no_add_no_remove_1():
         import a
         import b
         import c
+
         a, c, os, sys
     ''').strip()
     assert result == expected
@@ -259,6 +260,79 @@ def test_collect_exports_1():
 
     assert result == expected
 
+def test_collect_exports_module_1():
+    with tempfile.TemporaryDirectory() as d:
+        os.mkdir(os.path.join(d, 'test_mod'))
+        with open(os.path.join(d, 'test_mod', '__init__.py'), 'w') as f:
+            f.write(dedent('''
+                # test_mod/__init__.py
+
+                from .submod import e
+
+                _private = 0
+                a = 1
+                b = 2
+                c = 3
+                d = 4
+
+                print("The test_mod code is being executed")
+
+            ''').lstrip())
+        with open(os.path.join(d, 'test_mod', 'submod.py'), 'w') as f:
+            f.write(dedent('''
+                # test_mod/submod.py
+
+                e = 5
+                f = 6
+            ''').lstrip())
+
+        env = os.environ.copy()
+        env['PYTHONPATH'] = '.'
+        result = pipe([BIN_DIR+"/collect-exports", 'test_mod'], cwd=d, env=env)
+        # TODO: Make this work statically
+        expected = dedent('''
+            The test_mod code is being executed
+            from   test_mod                 import a, b, c, d, e
+        ''').strip()
+        assert result == expected
+
+
+def test_collect_exports_module_2():
+    with tempfile.TemporaryDirectory() as d:
+        os.mkdir(os.path.join(d, 'test_mod'))
+        with open(os.path.join(d, 'test_mod', '__init__.py'), 'w') as f:
+            f.write(dedent('''
+                # test_mod/__init__.py
+
+                __all__ = ['a', 'b', 'e']
+
+                from .submod import e
+
+                _private = 0
+                a = 1
+                b = 2
+                c = 3
+                d = 4
+
+                print("The test_mod code is being executed")
+
+            ''').lstrip())
+        with open(os.path.join(d, 'test_mod', 'submod.py'), 'w') as f:
+            f.write(dedent('''
+                # test_mod/submod.py
+
+                e = 5
+                f = 6
+            ''').lstrip())
+
+        env = os.environ.copy()
+        env['PYTHONPATH'] = '.'
+        result = pipe([BIN_DIR+"/collect-exports", 'test_mod'], cwd=d, env=env)
+
+        expected = dedent('''
+            from   test_mod                 import a, b, e
+        ''').strip()
+        assert result == expected
 
 def test_find_import_1():
     result = pipe([BIN_DIR+"/find-import", "np"])
@@ -279,8 +353,6 @@ def test_py_eval_1():
         [PYFLYBY] b64decode('aGVsbG8=')
         b'hello'
     """).strip()
-    if PY2:
-        expected = expected.replace("b'hello'", "'hello'")
     assert result == expected
 
 
@@ -291,8 +363,6 @@ def test_py_exec_1():
         [PYFLYBY] if 1: print(b64decode('aGVsbG8='))
         b'hello'
     """).strip()
-    if PY2:
-        expected = expected.replace("b'hello'", "hello")
     assert result == expected
 
 
@@ -341,6 +411,7 @@ def test_tidy_imports_query_no_change_1():
     input = dedent('''
         from __future__ import absolute_import, division
         import x1
+
         x1
     ''')
     with tempfile.NamedTemporaryFile(suffix=".py", mode='w+') as f:
@@ -378,6 +449,7 @@ def test_tidy_imports_query_y_1():
     expected = dedent("""
         from __future__ import absolute_import, division
         import x1
+
         x1
     """)
     assert output == expected
@@ -426,36 +498,6 @@ def test_tidy_imports_query_junk_1():
     assert output == input
 
 
-# Note, these tests will fail if the system does not have both python2 and
-# python3 in the PATH
-def test_tidy_imports_py2_fallback():
-    input = dedent('''
-        import x
-
-        def f(*args, x=1):
-            pass
-    ''')
-    with tempfile.NamedTemporaryFile(suffix=".py", mode='w+') as f:
-        f.write(input)
-        f.flush()
-        child = pexpect.spawn(python, [BIN_DIR+'/tidy-imports', "--py23-fallback", f.name], timeout=5.0)
-        child.logfile = BytesIO()
-        child.expect_exact(" [y/N]")
-        child.send("n\n")
-        child.expect(pexpect.EOF)
-        with open(f.name) as f2:
-            output = f2.read()
-    proc_output = child.logfile.getvalue()
-    assert b"removed unused 'import x'" in proc_output
-    assert output == input
-    if PY2:
-        assert b"SyntaxError detected" in proc_output, proc_output
-        assert b"falling back" in proc_output, proc_output
-    else:
-        assert b"SyntaxError detected" not in proc_output, proc_output
-        assert b"falling back" not in proc_output, proc_output
-
-
 @pytest.mark.skip(reason="seem to fail at importing six even if installed")
 def test_tidy_imports_py3_fallback():
     input = dedent('''
@@ -476,12 +518,9 @@ def test_tidy_imports_py3_fallback():
     proc_output = child.logfile.getvalue()
     assert b"removed unused 'import x'" in proc_output
     assert output == input
-    if PY3:
-        assert b"SyntaxError detected" in proc_output, proc_output
-        assert b"falling back" in proc_output, proc_output
-    else:
-        assert b"SyntaxError detected" not in proc_output, proc_output
-        assert b"falling back" not in proc_output, proc_output
+    assert b"SyntaxError detected" in proc_output, proc_output
+    assert b"falling back" in proc_output, proc_output
+
 
 def test_tidy_imports_symlinks_default():
     input = dedent('''
@@ -646,3 +685,147 @@ def test_tidy_imports_symlinks_bad_argument():
     assert b"error: --symlinks must be one of" in proc_output
     assert output == input
     assert symlink_output == input
+
+
+def test_debug_filetype_with_py():
+    with tempfile.NamedTemporaryFile(suffix=".py", mode='w+') as f:
+        f.write(dedent("""
+            sys.argv
+        """).lstrip())
+        f.flush()
+        command = [BIN_DIR+"/py", "--debug", f.name]
+
+        output_result = b""
+        child = pexpect.spawn(' '.join(command), timeout=5)
+        child.expect('ipdb>')
+        output_result += child.before
+        child.sendline('c')
+        output_result += child.before
+        child.expect(pexpect.EOF)
+        output_result += child.before
+
+        expected = "Entering debugger.  Use 'n' to step, 'c' to run, 'q' to stop."
+        assert expected in output_result.decode()
+
+
+def test_tidy_imports_sorting():
+    with tempfile.NamedTemporaryFile(suffix=".py", mode="w+") as f:
+        f.write(
+            dedent(
+                """
+            from very_very_very_long_named_package.very_long_named_module \
+                                import(small_class, very_large_named_class as small_name)
+            import numpy
+
+            from pkg1.mod1 import foo
+            from pkg1.mod2 import bar
+            from pkg2 import baz
+            import yy
+
+            from pkg1.mod1 import foo2
+            from pkg1.mod3 import quux
+            from pkg2 import baar
+            import sympy
+            import zz
+
+
+            zz.foo()
+            bar()
+            quux()
+            foo2()
+            yy.f()
+            bar()
+            foo()
+            numpy.arange()
+            baz
+            baar
+            sympy
+            small_class
+            small_name
+        """
+            ).lstrip()
+        )
+        f.flush()
+        result = pipe([BIN_DIR + "/tidy-imports", f.name])
+        expected = (
+            dedent(
+                """
+            import numpy
+
+            from   pkg1.mod1                import foo, foo2
+            from   pkg1.mod2                import bar
+            from   pkg1.mod3                import quux
+
+            from   pkg2                     import baar, baz
+            import sympy
+            from   very_very_very_long_named_package.very_long_named_module \\
+                                            import (small_class,
+                                                    very_large_named_class as small_name)
+            import yy
+            import zz
+
+            zz.foo()
+            bar()
+            quux()
+            foo2()
+            yy.f()
+            bar()
+            foo()
+            numpy.arange()
+            baz
+            baar
+            sympy
+            small_class
+            small_name
+        """
+            )
+            .strip()
+            .format(f=f)
+        )
+        assert result == expected
+
+
+def test_tidy_imports_forward_references():
+    with tempfile.TemporaryDirectory() as temp_dir:
+        foo = os.path.join(temp_dir, "foo.py")
+        with open(foo, "w") as foo_fp:
+            foo_fp.write(dedent("""
+                from __future__ import annotations
+
+
+                class A:
+                    param1: str
+                    param2: B
+
+
+                class B:
+                    param1: str
+            """).lstrip())
+            foo_fp.flush()
+
+        dot_pyflyby = os.path.join(temp_dir, ".pyflyby")
+        with open(dot_pyflyby, "w") as dot_pyflyby_fp:
+            dot_pyflyby_fp.write(dedent("""
+                from foo import A, B
+            """).lstrip())
+            dot_pyflyby_fp.flush()
+        with CwdCtx(temp_dir):
+            result = pipe(
+                [BIN_DIR + "/tidy-imports", foo_fp.name],
+                env={"PYFLYBY_PATH": dot_pyflyby},
+            )
+
+            expected = dedent(
+                """
+                from __future__ import annotations
+
+                class A:
+                    param1: str
+                    param2: B
+
+
+                class B:
+                    param1: str
+            """
+            ).strip()
+        assert result == expected
