@@ -3,12 +3,10 @@
 # License for THIS FILE ONLY: CC0 Public Domain Dedication
 # http://creativecommons.org/publicdomain/zero/1.0/
 
-from __future__ import (absolute_import, division, print_function,
-                        with_statement)
+
 
 import ast
 
-from   six                      import PY3
 
 import pytest
 import warnings
@@ -118,17 +116,6 @@ def test_CompilerFlags_from_ast_1():
         assert result == CompilerFlags(0x18000)
 
 
-@pytest.mark.skipif(
-    PY3,
-    reason="print function is not invalid syntax in Python 3.")
-def test_CompilerFlags_compile_1():
-    # Should raise SyntaxError:
-    with pytest.raises(SyntaxError):
-        compile("print('x', file=None)", "?", "exec", flags=0, dont_inherit=1)
-    # Shouldn't raise SyntaxError:
-    compile("print('x', file=None)", "?", "exec", flags=CompilerFlags("print_function"), dont_inherit=1)
-
-
 def test_CompilerFlags_bad_name_1():
     with pytest.raises(ValueError):
         CompilerFlags("print_statement")
