@@ -1,18 +1,16 @@
 # pyflyby/test_imports2s.py
+import tempfile
 
 # License for THIS FILE ONLY: CC0 Public Domain Dedication
 # http://creativecommons.org/publicdomain/zero/1.0/
 
-from __future__ import (absolute_import, division, print_function,
-                        with_statement)
+
 
 import pytest
 import sys
 from   textwrap                 import dedent
 import types
 
-from   six                      import PY2
-
 from   pyflyby._importdb        import ImportDB
 from   pyflyby._imports2s       import (canonicalize_imports,
                                         fix_unused_and_missing_imports,
@@ -825,15 +823,21 @@ def test_replace_star_no_imports_found(capsys):
     assert 'Traceback' not in captured.err
 
 def test_replace_star_imports_1():
-    m = types.ModuleType("fake_test_module_345489")
-    m.__all__ = ['f1', 'f2', 'f3', 'f4', 'f5']
-    sys.modules["fake_test_module_345489"] = m
     input = PythonBlock(dedent('''
         from mod1                    import f1
         from fake_test_module_345489 import *
         from mod2                    import f5
     ''').lstrip(), filename="/foo/test_replace_star_imports_1.py")
-    output = replace_star_imports(input)
+    m = types.ModuleType("fake_test_module_345489")
+    m.__all__ = ['f1', 'f2', 'f3', 'f4', 'f5']
+    sys.modules["fake_test_module_345489"] = m
+
+    with tempfile.NamedTemporaryFile(suffix=".py", mode='w+') as f:
+        f.write(f"__all__ = {m.__all__}")
+        m.__file__ = f.name
+        f.flush()
+        output = replace_star_imports(input)
+
     expected = PythonBlock(dedent('''
         from fake_test_module_345489 import f1, f2, f3, f4
         from mod2                    import f5
@@ -881,26 +885,39 @@ def test_transform_imports_1():
 
 
 def test_canonicalize_imports_1():
-    input = PythonBlock(dedent('''
+    input = PythonBlock(
+        dedent(
+            """
+        from very_very_very_long_named_package.very_long_named_module \\
+                                import(small_class, very_large_named_class as small_name)
         from m import x
         from m import x as X
         import m.x
         print(m.x, m.xx)
-    ''').lstrip(), filename="/foo/test_transform_imports_1.py")
-    db = ImportDB("""
+    """
+        ).lstrip(),
+        filename="/foo/test_transform_imports_1.py",
+    )
+    db = ImportDB(
+        """
         __canonical_imports__ = {"m.x": "m.y.z"}
-    """)
+    """
+    )
     output = canonicalize_imports(input, db=db)
-    expected = PythonBlock(dedent('''
+    expected = PythonBlock(
+        dedent("""
         import m.y.z
-        from m.y import z as X, z as x
+        from m.y                                                      import (z as X,
+                                                                              z as x)
+        from very_very_very_long_named_package.very_long_named_module import (small_class,
+                                                                              very_large_named_class as small_name)
         print(m.y.z, m.xx)
-    ''').lstrip(), filename="/foo/test_transform_imports_1.py")
+        """).lstrip(),
+        filename="/foo/test_transform_imports_1.py",
+    )
     assert output == expected
 
-@pytest.mark.skipif(
-    PY2,
-    reason="Python 3-only syntax.")
+
 def test_canonicalize_imports_f_string_1():
     input = PythonBlock(dedent('''
         a = 1
@@ -965,25 +982,6 @@ def test_with_1():
     assert expected == output
 
 
-@pytest.mark.skipif(
-    sys.version_info < (2,7),
-    reason="Old Python doesn't support multiple context managers")
-def test_with_multi_1():
-    input = PythonBlock(dedent('''
-        with       aa as xx  , bb as yy, cc as zz:
-            pass
-    ''').lstrip())
-    db = ImportDB("from M import aa, bb, cc, dd, xx, yy, zz")
-    output = fix_unused_and_missing_imports(input, db=db)
-    expected = PythonBlock(dedent('''
-        from M import aa, bb, cc
-
-        with       aa as xx  , bb as yy, cc as zz:
-            pass
-    ''').lstrip())
-    assert expected == output
-
-
 def test_fix_unused_imports_repeated_1():
     input = PythonBlock(dedent('''
         import foo1, foo2, foo1, foo1
