@@ -3,8 +3,7 @@
 # License for THIS FILE ONLY: CC0 Public Domain Dedication
 # http://creativecommons.org/publicdomain/zero/1.0/
 
-from __future__ import (absolute_import, division, print_function,
-                        with_statement)
+
 
 from   pyflyby._flags           import CompilerFlags
 from   pyflyby._importstmt      import Import, ImportSplit, ImportStatement
